local U = require ("__sc-redux__.util")
local C = require ("__sc-redux__.constants")
local A = require ("__sc-redux__.anim")
local scdata = require ("scdata")
local add_shield_prototype = require ("prototypes.entity.effect.shield")

local function add_dragoon_based_prototype(unit_name, level)
  tierdata = C.TIERDATA[level]
  hm = tierdata.health
  am = tierdata.attack
  tint = tierdata.tint
  uscdat = scdata.unit[unit_name]

  data:extend(
  {
    {
      type = "unit",
      name = string.format("protoss-%s-%02d", unit_name, level),
      icon = string.format("__protoss-redux__/graphics/dragoon/icon-%02d.png", level),
      icon_size = 64, icon_mipmaps = 4,
      flags = {"placeable-player", "placeable-enemy", "placeable-off-grid"},
      collision_mask = { "object-layer" },
      max_health = hm * (uscdat.health + uscdat.shield),
      order = string.format("p-b-b-%02d", level),
      subgroup = "enemies",
      resistances = {},
      healing_per_tick = hm * uscdat.shield / (uscdat.health + uscdat.shield) * scdata.shield_regen,
      collision_box =
      {
        {-uscdat.collision[1]/32, -uscdat.collision[2]/32},
        {uscdat.collision[1]/32, uscdat.collision[2]/32},
      },
      selection_box =
      {
        {-uscdat.collision[1]/32, -uscdat.collision[2]/32},
        {uscdat.collision[1]/32, uscdat.collision[2]/32},
      },
      attack_parameters =
      {
        type = "projectile",
        range = 2 * uscdat.range,
        cooldown = uscdat.cooldown / 0.4,
        cooldown_deviation = 0.05,
        warmup = 17,
        ammo_type =
        {
          category = "biological",
          target_type = "entity",
          action =
          {
            type = "direct",
            action_delivery =
            {
              type = "projectile",
              projectile = string.format("protoss-%s-bullet-%02d", unit_name, level),
              starting_speed = 0.4
            }
          }
        },
        sound = { filename = "__protoss-redux__/sound/common/dragbull.wav" },
        animation =
        {
          layers =
          {
            A.dragoon_attack(),
            A.dragoon_attack_mask(tint),
            A.dragoon_attack_light(),
            A.dragoon_attack_shadow()
          }
        },
        range_mode = "center-to-center"
      },
      alternative_attacking_frame_sequence =
      {
        warmup_frame_sequence = { 1, 2, 3, 4, 5 },
        warmup2_frame_sequence = { 6, 6, 6, 6, 6 },
        attacking_frame_sequence = { 7, 8, 7 },
        cooldown_frame_sequence = { 6 },
        prepared_frame_sequence = { 6 },
        back_to_walk_frame_sequence = { 5, 4, 3, 2, 1 },
        warmup_animation_speed = 1 / 5 * 0.4,
        attacking_animation_speed = 1 / 3 * 0.4,
        cooldown_animation_speed = 0.4,
        prepared_animation_speed = 0.4,
        back_to_walk_animation_speed = 1 / 5 * 0.4
      },
      vision_distance = 30,
      movement_speed = uscdat.movement * C.MOVEMENT_MULTIPLIER,
      distance_per_frame = 12 / 160, -- check on gimp per frame movement / 160
      pollution_to_join_attack = 4,
      distraction_cooldown = 300,
      min_pursue_time = 10 * 60,
      max_pursue_distance = 50,
      corpse =
      {
        string.format("protoss-%s-die-%02d", unit_name, level),
        string.format("protoss-%s-corpse-%02d", unit_name, level),
      },
      dying_sound = { filename = "__protoss-redux__/sound/dragoon/pdrdth00.wav" },
      run_animation =
      {
        layers =
        {
          A.dragoon_run(),
          A.dragoon_run_mask(tint),
          A.dragoon_run_light(),
          A.dragoon_run_shadow()
        }
      },
      ai_settings = { destroy_when_commands_fail = true, allow_try_return_to_spawner = true },
    },
    {
      type = "projectile",
      name = string.format("protoss-%s-bullet-%02d", unit_name, level),
      flags = {"not-on-map"},
      acceleration = 0,
      animation = A.projectiles_phase_disruptor(),
      height = 0.25,
      action =
      {
        type = "direct",
        action_delivery =
        {
          type = "instant",
          target_effects =
          {
            {
              type = "damage",
              damage = { amount = am * uscdat.damage , type = "physical"}
            },
            {
              type = "create-entity",
              entity_name = "protoss-explosion-medium"
            },
            {
              type = "play-sound",
              sound = { filename = "__protoss-redux__/sound/common/explomed.wav" },
            }
          }
        }
      }
    },
    {
      type = "corpse",
      name = string.format("protoss-%s-die-%02d", unit_name, level),
      flags = { "not-on-map" },
      subgroup = "corpses",
      animation = 
      {
        filename = "__protoss-redux__/graphics/dragoon/die.png",
        width = 384,
        height = 308,
        frame_count = 7,
        direction_count = 1,
        scale = 0.5,
      },
      dying_speed = 0.4 / 7,
      time_before_shading_off = 0,
      time_before_removed = math.ceil(7 / 0.4)
    },
    {
      type = "corpse",
      name = string.format("protoss-%s-corpse-%02d", unit_name, level),
      flags = { "not-on-map" },
      subgroup = "corpses",
      animation = 
      {
        filename = "__protoss-redux__/graphics/dragoon/corpse.png",
        line_length = 3,
        width = 366,
        height = 348,
        frame_count = 6,
        frame_sequence = U.corpse_frame_sequence(7, 5),
        direction_count = 1,
        scale = 0.5
      },
      dying_speed = 0.4 / U.calc_corpse_time(7, 5),
      time_before_shading_off = 0,
      time_before_removed = math.ceil(U.calc_corpse_time(7, 5) / 0.4) 
    }
  })
end

for i = 1, 4 do add_dragoon_based_prototype("dragoon", i) end
add_shield_prototype("dragoon")

