local C = require ("__sc-redux__.constants")
local scdata = require ("scdata")
local add_shield_prototype = require ("prototypes.entity.effect.shield")

local function add_zealot_based_prototype(unit_name, level)
  tierdata = C.TIERDATA[level]
  hm = tierdata.health
  am = tierdata.attack
  tint = tierdata.tint
  uscdat = scdata.unit[unit_name]

  data:extend(
  {
    {
      type = "unit",
      name = string.format("protoss-%s-%02d", unit_name, level),
      icon = string.format("__protoss-redux__/graphics/icons/zealot-%02d.png", level),
      icon_size = 64, icon_mipmaps = 4,
      flags = {"placeable-player", "placeable-enemy", "placeable-off-grid"},
      collision_mask = { "object-layer" },
      max_health = hm * 160,
      order = string.format("p-b-a-%02d", level),
      subgroup = "enemies",
      resistances = {},
      healing_per_tick = hm * uscdat.shield / (uscdat.health + uscdat.shield) * scdata.shield_regen,
      collision_box =
      {
        {-math.max(unpack(uscdat.collision))/32, -math.max(unpack(uscdat.collision))/32},
        {math.max(unpack(uscdat.collision))/32, math.max(unpack(uscdat.collision))/32},
      },
      selection_box =
      {
        {-math.max(unpack(uscdat.collision))/32, -math.max(unpack(uscdat.collision))/32},
        {math.max(unpack(uscdat.collision))/32, math.max(unpack(uscdat.collision))/32},
      },
      attack_parameters =
      {
        type = "projectile",
        range = 2 * uscdat.range,
        cooldown = uscdat.cooldown / 0.4,
        cooldown_deviation = 0.05,
        warmup = 11,
        ammo_type =
        {
          category = "melee",
          target_type = "entity",
          action = {
            type = "direct",
            action_delivery =
            {
              type = "instant",
              target_effects =
              {
                {
                  type = "damage",
                  repeat_count = uscdat.multi,
                  damage = { amount = uscdat.damage * am, type = "physical"}
                },
                {
                  type = "play-sound",
                  sound = { filename = "__protoss-redux__/sound/zealot/pzehit00.wav" }
                }
              }
            }
          }
        },
        sound =
        {
          aggregation = { max_count = 1, remove = true },
          variations =
          {
            { filename = "__protoss-redux__/sound/zealot/pzeatt00.wav" },
            { filename = "__protoss-redux__/sound/zealot/pzeatt01.wav" }
          }
        },
        animation =
        {
          layers =
          {
            {
              filenames =
              {
                "__protoss-redux__/graphics/zealot/attack-01.png",
                "__protoss-redux__/graphics/zealot/attack-02.png",
                "__protoss-redux__/graphics/zealot/attack-03.png",
                "__protoss-redux__/graphics/zealot/attack-04.png",
              },
              slice = 5,
              lines_per_file = 8,
              size = { 360, 306 },
              frame_count = 5,
              direction_count = 32,
              animation_speed = 0.4,
              scale = 0.5,
            },
            {
              filenames =
              {
                "__protoss-redux__/graphics/zealot/attack-mask-01.png",
                "__protoss-redux__/graphics/zealot/attack-mask-02.png",
                "__protoss-redux__/graphics/zealot/attack-mask-03.png",
                "__protoss-redux__/graphics/zealot/attack-mask-04.png",
              },
              slice = 5,
              lines_per_file = 8,
              flags = { "mask" },
              size = { 360, 306 },
              frame_count = 5,
              direction_count = 32,
              animation_speed = 0.4,
              scale = 0.5,
              tint = tint
            },
            {
              filenames =
              {
                "__protoss-redux__/graphics/zealot/attack-light-01.png",
                "__protoss-redux__/graphics/zealot/attack-light-02.png",
                "__protoss-redux__/graphics/zealot/attack-light-03.png",
                "__protoss-redux__/graphics/zealot/attack-light-04.png",
              },
              slice = 5,
              lines_per_file = 8,
              draw_as_glow = true,
              flags = { "light" },
              size = { 360, 306 },
              frame_count = 5,
              direction_count = 32,
              animation_speed = 0.4,
              scale = 0.5,
            },
            {
              filenames =
              {
                "__protoss-redux__/graphics/zealot/attack-shadow-01.png",
                "__protoss-redux__/graphics/zealot/attack-shadow-02.png",
                "__protoss-redux__/graphics/zealot/attack-shadow-03.png",
                "__protoss-redux__/graphics/zealot/attack-shadow-04.png",
              },
              slice = 5,
              lines_per_file = 8,
              draw_as_shadow = true,
              size = { 252, 250 },
              frame_count = 5,
              direction_count = 32,
              animation_speed = 0.4,
              scale = 0.5,
            },
          }
        },
        range_mode = "bounding-box-to-bounding-box"
      },
      alternative_attacking_frame_sequence =
      {
        warmup_frame_sequence = { 1 },
        warmup2_frame_sequence = { 1 },
        attacking_frame_sequence = { 2, 3, 4, 5, 4, 3, 2 },
        cooldown_frame_sequence = { 1 },
        prepared_frame_sequence = { 1 },
        back_to_walk_frame_sequence = { 1 },
        warmup_animation_speed = 1,
        attacking_animation_speed = 1 / 7 * 0.4,
        cooldown_animation_speed = 1,
        prepared_animation_speed = 1,
        back_to_walk_animation_speed = 1
      },
      vision_distance = 30,
      movement_speed = uscdat.movement * C.MOVEMENT_MULTIPLIER,
      distance_per_frame = 14 / 160, -- check on gimp per frame movement / 80
      pollution_to_join_attack = 4,
      distraction_cooldown = 300,
      min_pursue_time = 10 * 60,
      max_pursue_distance = 50,
      corpse = string.format("protoss-%s-die-%02d", unit_name, level),
      dying_sound = { filename = "__protoss-redux__/sound/zealot/pzedth00.wav" },
      run_animation =
      {
        layers =
        {
          {
            filenames =
            {
              "__protoss-redux__/graphics/zealot/run-01.png",
              "__protoss-redux__/graphics/zealot/run-02.png",
              "__protoss-redux__/graphics/zealot/run-03.png",
              "__protoss-redux__/graphics/zealot/run-04.png",
            },
            slice = 8,
            lines_per_file = 8,
            size = { 136, 170 },
            frame_count = 8,
            direction_count = 32,
            animation_speed = 0.4,
            scale = 0.5,
          },
          {
            filenames =
            {
              "__protoss-redux__/graphics/zealot/run-mask-01.png",
              "__protoss-redux__/graphics/zealot/run-mask-02.png",
              "__protoss-redux__/graphics/zealot/run-mask-03.png",
              "__protoss-redux__/graphics/zealot/run-mask-04.png",
            },
            slice = 8,
            lines_per_file = 8,
            flags = { "mask" },
            size = { 136, 170 },
            frame_count = 8,
            direction_count = 32,
            animation_speed = 0.4,
            scale = 0.5,
            tint = tint
          },
          {
            filenames =
            {
              "__protoss-redux__/graphics/zealot/run-light-01.png",
              "__protoss-redux__/graphics/zealot/run-light-02.png",
              "__protoss-redux__/graphics/zealot/run-light-03.png",
              "__protoss-redux__/graphics/zealot/run-light-04.png",
            },
            slice = 8,
            lines_per_file = 8,
            draw_as_glow = true,
            flags = { "light" },
            size = { 136, 170 },
            frame_count = 8,
            direction_count = 32,
            animation_speed = 0.4,
            scale = 0.5,
          },
          {
            filenames =
            {
              "__protoss-redux__/graphics/zealot/run-shadow-01.png",
              "__protoss-redux__/graphics/zealot/run-shadow-02.png",
              "__protoss-redux__/graphics/zealot/run-shadow-03.png",
              "__protoss-redux__/graphics/zealot/run-shadow-04.png",
            },
            slice = 8,
            lines_per_file = 8,
            draw_as_shadow = true,
            size = { 136, 190 },
            frame_count = 8,
            direction_count = 32,
            animation_speed = 0.4,
            scale = 0.5,
          },
        }
      },
      ai_settings = { destroy_when_commands_fail = true, allow_try_return_to_spawner = true },
    },
    {
      type = "corpse",
      name = string.format("protoss-%s-die-%02d", unit_name, level),
      flags = {"placeable-off-grid", "not-repairable", "not-on-map"},
      subgroup = "corpses",
      animation = 
      {
        layers =
        {
          {
            filename = "__protoss-redux__/graphics/zealot/die.png",
            width = 272,
            height = 430,
            frame_count = 7,
            direction_count = 1,
            scale = 0.5,
          },
          {
            filename = "__protoss-redux__/graphics/zealot/die-mask.png",
            flags = { "mask" },
            width = 272,
            height = 430,
            frame_count = 7,
            direction_count = 1,
            scale = 0.5,
            tint = tint
          },
          {
            filename = "__protoss-redux__/graphics/zealot/die-light.png",
            draw_as_glow = true,
            flags = { "light" },
            width = 272,
            height = 430,
            frame_count = 7,
            direction_count = 1,
            scale = 0.5,
          }
        }
      },
      dying_speed = 0.4 / 7,
      time_before_shading_off = 0,
      time_before_removed = math.ceil(7 / 0.4)
    },
  })
end

for i = 1, 4 do add_zealot_based_prototype("zealot", i) end
for i = 1, 4 do add_zealot_based_prototype("speedlot", i) end

add_shield_prototype("zealot")
add_shield_prototype("speedlot")

